/***************************************************************************
         Copyright (c) Microsoft Corporation, All rights reserved.             
    This code sample is provided "AS IS" without warranty of any kind, 
    it is not recommended for use in a production environment.
***************************************************************************/

/*---------------------------------------------------------
 String function library.
---------------------------------------------------------*/
#include "project.h"
#include <windows.h>
#include "text.h"

UniStr  uniCopy( UniStr dest, CUniStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return dest;
    if (maxLen < 0) maxLen = uniLen(src);
    if (maxLen <= 0) return dest;

    wcsncpy( dest, src, maxLen );
    dest[maxLen] = UNI('\0');
    return dest;
}

AnsiStr  ansiCopy( AnsiStr dest, CAnsiStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return dest;
    if (maxLen < 0) maxLen = ansiLen(src);
    if (maxLen <= 0) return dest;

    strncpy( dest, src, maxLen );
    dest[maxLen] = '\0';
    return dest;
}

UniStr  uniCat( UniStr dest, CUniStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return dest;
    
    int len = uniLen(dest);
    if (maxLen < 0)  maxLen = len + uniLen(src);
    if (maxLen <= 0 || maxLen <= len) return dest;

    wcsncat( dest, src, maxLen - len );
    dest[maxLen] = UNI('\0');
    return dest;
}

AnsiStr  ansiCat( AnsiStr dest, CAnsiStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return dest;
    
    int len = ansiLen(dest);
    if (maxLen < 0)  maxLen = len + ansiLen(src);
    if (maxLen <= 0 || maxLen <= len) return dest;

    strncat( dest, src, maxLen - len );
    dest[maxLen] = '\0';
    return dest;
}


int ansiToUniLen( CAnsiStr s )
{
    if (s == NULL)
        return 0;
    return (int)mbstowcs( NULL, s, (size_t)(-1) );
}

int uniToAnsiLen( CUniStr s )
{
    if (s == NULL)
        return 0;
    return (int)wcstombs( NULL, s, (size_t)(-1) );
}

UniStr ansiToUniCopy( UniStr dest, CAnsiStr src, int maxLen )
{
    if (!dest) return NULL;
    dest[0] = UNI('\0');
    if (!src || maxLen <= 0)  return dest;
    
    mbstowcs( dest, src, maxLen+1 );
    dest[maxLen] = UNI('\0');
    return dest;
}

AnsiStr uniToAnsiCopy( AnsiStr dest, CUniStr src, int maxLen )
{
    if (!dest) return NULL;
    dest[0] = '\0';
    if (!src || maxLen <= 0)  return dest;

    wcstombs( dest, src, maxLen+1 );
    dest[maxLen] = '\0';
    return dest;
}

UniStr  ansiToUniCat( UniStr dest, CAnsiStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return dest;

    int len = uniLen(dest);
    ansiToUniCopy( dest+len, src, maxLen - len );
    return dest;
}

AnsiStr uniToAnsiCat( AnsiStr dest, CUniStr src, int maxLen )
{
    if (!dest) return NULL;
    if (!src)  return NULL;

    int len = ansiLen(dest);
    uniToAnsiCopy( dest+len, src, maxLen - len );
    return dest;
}


UniStr ansiToUniDup( CAnsiStr s )
{
    if (!s) return NULL;

    int    len  = ansiToUniLen(s);
    UniStr dest = (UniStr) VSAlloc ( (len+1)*sizeof(UniChar) );

    return ansiToUniCopy( dest, s, len );
}

AnsiStr uniToAnsiDup( CUniStr s )
{
    if (!s) return NULL;

    int     len  = uniToAnsiLen(s);
    AnsiStr dest = (AnsiStr) VSAlloc( (len+1)*sizeof(AnsiChar) );

    return uniToAnsiCopy( dest, s, len );
}



void bstrFree( BSTR& bstr )
{
    if (bstr) SysFreeString(bstr);
    bstr = NULL;
}

BSTR bstrCopy( BSTR& bstr, COleStr src, int maxLen )
{
    if (maxLen<0)  maxLen = oleLen(src); 
    if (maxLen<=0) return NULL;
    
    if (bstr) 
      SysReAllocStringLen( &bstr, src, maxLen );
    else
      bstr = SysAllocStringLen( src, maxLen );

    return bstr;
}

BSTR bstrCat( BSTR& bstr, COleStr ostr, int maxLen )
{
    if (!ostr) return bstr;
    if (maxLen < 0) maxLen = oleLen(ostr);
    if (maxLen <= 0) return bstr;

    int bstrlen = bstrLen(bstr);

    BSTR temp = SysAllocStringLen( NULL, 
                    bstrlen + maxLen + 1 );
    if (!temp) return bstr;

    oleCopy( temp, bstr, bstrlen );
    oleCat( temp, ostr, maxLen );
    bstrFree(bstr);
    bstr = temp;

    return bstr;
}

